@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -26,7 +26,7 @@ import hypothesis.internal.conjecture.utils as cu
 from hypothesis import Verbosity
 from hypothesis._settings import note_deprecation
 from hypothesis.errors import InvalidArgument
-from hypothesis.internal.compat import hrange, text_type
+from hypothesis.internal.compat import hrange, integer_types, text_type
 from hypothesis.internal.coverage import check_function
 from hypothesis.internal.reflection import proxies
 from hypothesis.internal.validation import check_type
@@ -126,13 +126,9 @@ class ArrayStrategy(SearchStrategy):
     def __init__(self, element_strategy, shape, dtype, fill, unique):
         self.shape = tuple(shape)
         self.fill = fill
+        assert shape, "Zero-dimensional array shape is special-cased in arrays()"
         check_argument(
-            shape,
-            "Array shape must have at least one dimension, provided shape was {}",
-            shape,
-        )
-        check_argument(
-            all(isinstance(s, int) for s in shape),
+            all(isinstance(s, integer_types) for s in shape),
             "Array shape must be integer in each dimension, provided shape was {}",
             shape,
         )
@@ -176,7 +172,7 @@ class ArrayStrategy(SearchStrategy):
                 "dtype %r - instead it becomes %r .  Consider using a more "
                 "precise strategy, as this will be an error in a future "
                 "version." % (val, strategy, self.dtype, result[idx]),
-                since="2018-07-31",
+                since="2018-10-25",
             )
             # Because the message includes the value of the generated element,
             # it would be easy to spam users with thousands of warnings.
@@ -393,8 +389,7 @@ def arrays(
         shape = (shape,)
     shape = tuple(shape)
     if not shape:
-        if dtype.kind != u"O":
-            return draw(elements)
+        return np.full(shape=(), fill_value=draw(elements), dtype=dtype)
     fill = fill_for(elements=elements, unique=unique, fill=fill)
     return draw(ArrayStrategy(elements, shape, dtype, fill, unique))
 
@@ -430,7 +425,20 @@ def defines_dtype_strategy(strat):
     @st.defines_strategy
     @proxies(strat)
     def inner(*args, **kwargs):
-        return strat(*args, **kwargs).map(np.dtype)
+        strategy = strat(*args, **kwargs)
+
+        def convert_to_dtype(x):
+            """Helper to debug issue #1798."""
+            try:
+                return np.dtype(x)
+            except ValueError:
+                print(
+                    "Got invalid dtype value=%r from strategy=%r, function=%r"
+                    % (x, strategy, strat)
+                )
+                raise
+
+        return strategy.map(convert_to_dtype)
 
     return inner
 
